@@ -301,9 +301,9 @@ fn to_lsp_diagnostic(
             severity,
             tags,
             code,
-            code_description: diagnostic.to_ruff_url().and_then(|url| {
+            code_description: diagnostic.documentation_url().and_then(|url| {
                 Some(lsp_types::CodeDescription {
-                    href: lsp_types::Url::parse(&url).ok()?,
+                    href: lsp_types::Url::parse(url).ok()?,
                 })
             }),
             source: Some(DIAGNOSTIC_NAME.into()),
