@@ -68,6 +68,134 @@ mod tests {
         ");
     }
 
+    #[test]
+    fn goto_type_of_typing_dot_literal() {
+        let test = cursor_test(
+            r#"
+            from typing import Literal
+
+            a<CURSOR>b = Literal
+            "#,
+        );
+
+        assert_snapshot!(test.goto_type_definition(), @r"
+        info[goto-type-definition]: Type definition
+           --> stdlib/typing.pyi:351:1
+            |
+        349 | Final: _SpecialForm
+        350 |
+        351 | Literal: _SpecialForm
+            | ^^^^^^^
+        352 | TypedDict: _SpecialForm
+            |
+        info: Source
+         --> main.py:4:1
+          |
+        2 | from typing import Literal
+        3 |
+        4 | ab = Literal
+          | ^^
+          |
+        ");
+    }
+
+    // this is a slightly different case to the one above,
+    // since `Any` is a class in typeshed rather than a variable
+    #[test]
+    fn goto_type_of_typing_dot_any() {
+        let test = cursor_test(
+            r#"
+            from typing import Any
+
+            a<CURSOR>b = Any
+            "#,
+        );
+
+        assert_snapshot!(test.goto_type_definition(), @r#"
+        info[goto-type-definition]: Type definition
+           --> stdlib/typing.pyi:166:7
+            |
+        164 | # from _typeshed import AnnotationForm
+        165 |
+        166 | class Any:
+            |       ^^^
+        167 |     """Special type indicating an unconstrained type.
+            |
+        info: Source
+         --> main.py:4:1
+          |
+        2 | from typing import Any
+        3 |
+        4 | ab = Any
+          | ^^
+          |
+        "#);
+    }
+
+    // Similarly, `Generic` is a `type[]` type in typeshed
+    #[test]
+    fn goto_type_of_typing_dot_generic() {
+        let test = cursor_test(
+            r#"
+            from typing import Generic
+
+            a<CURSOR>b = Generic
+            "#,
+        );
+
+        assert_snapshot!(test.goto_type_definition(), @r"
+        info[goto-type-definition]: Type definition
+           --> stdlib/typing.pyi:770:1
+            |
+        768 |         def __class_getitem__(cls, args: TypeVar | tuple[TypeVar, ...]) -> _Final: ...
+        769 |
+        770 | Generic: type[_Generic]
+            | ^^^^^^^
+        771 |
+        772 | class _ProtocolMeta(ABCMeta):
+            |
+        info: Source
+         --> main.py:4:1
+          |
+        2 | from typing import Generic
+        3 |
+        4 | ab = Generic
+          | ^^
+          |
+        ");
+    }
+
+    #[test]
+    fn goto_type_of_ty_extensions_special_form() {
+        let test = cursor_test(
+            r#"
+            from ty_extensions import AlwaysTruthy
+
+            a<CURSOR>b = AlwaysTruthy
+            "#,
+        );
+
+        assert_snapshot!(test.goto_type_definition(), @r"
+        info[goto-type-definition]: Type definition
+          --> stdlib/ty_extensions.pyi:21:1
+           |
+        19 | # Types
+        20 | Unknown = object()
+        21 | AlwaysTruthy = object()
+           | ^^^^^^^^^^^^
+        22 | AlwaysFalsy = object()
+           |
+        info: Source
+         --> main.py:4:1
+          |
+        2 | from ty_extensions import AlwaysTruthy
+        3 |
+        4 | ab = AlwaysTruthy
+          | ^^
+          |
+        ");
+    }
+
     #[test]
     fn goto_type_of_expression_with_function_type() {
         let test = cursor_test(
@@ -199,13 +327,13 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:911:7
+           --> stdlib/builtins.pyi:915:7
             |
-        910 | @disjoint_base
-        911 | class str(Sequence[str]):
+        914 | @disjoint_base
+        915 | class str(Sequence[str]):
             |       ^^^
-        912 |     """str(object='') -> str
-        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        916 |     """str(object='') -> str
+        917 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
          --> main.py:4:1
@@ -227,13 +355,13 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:911:7
+           --> stdlib/builtins.pyi:915:7
             |
-        910 | @disjoint_base
-        911 | class str(Sequence[str]):
+        914 | @disjoint_base
+        915 | class str(Sequence[str]):
             |       ^^^
-        912 |     """str(object='') -> str
-        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        916 |     """str(object='') -> str
+        917 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
          --> main.py:2:10
@@ -276,10 +404,20 @@ mod tests {
             "#,
         );
 
-        // TODO: Goto type definition currently doesn't work for type param specs
-        // because the inference doesn't support them yet.
-        // This snapshot should show a single target pointing to `T`
-        assert_snapshot!(test.goto_type_definition(), @"No type definitions found");
+        assert_snapshot!(test.goto_type_definition(), @r"
+        info[goto-type-definition]: Type definition
+         --> main.py:2:14
+          |
+        2 | type Alias[**P = [int, str]] = Callable[P, int]
+          |              ^
+          |
+        info: Source
+         --> main.py:2:41
+          |
+        2 | type Alias[**P = [int, str]] = Callable[P, int]
+          |                                         ^
+          |
+        ");
     }
 
     #[test]
@@ -308,26 +446,8 @@ mod tests {
             "#,
         );
 
-        assert_snapshot!(test.goto_type_definition(), @r#"
-        info[goto-type-definition]: Type definition
-         --> main.py:4:1
-          |
-        2 | from typing_extensions import TypeAliasType
-        3 |
-        4 | Alias = TypeAliasType("Alias", tuple[int, int])
-          | ^^^^^
-        5 |
-        6 | Alias
-          |
-        info: Source
-         --> main.py:6:1
-          |
-        4 | Alias = TypeAliasType("Alias", tuple[int, int])
-        5 |
-        6 | Alias
-          | ^^^^^
-          |
-        "#);
+        // TODO: This should jump to the definition of `Alias` above.
+        assert_snapshot!(test.goto_type_definition(), @"No type definitions found");
     }
 
     #[test]
@@ -342,13 +462,13 @@ mod tests {
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:911:7
+           --> stdlib/builtins.pyi:915:7
             |
-        910 | @disjoint_base
-        911 | class str(Sequence[str]):
+        914 | @disjoint_base
+        915 | class str(Sequence[str]):
             |       ^^^
-        912 |     """str(object='') -> str
-        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        916 |     """str(object='') -> str
+        917 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
          --> main.py:4:6
@@ -376,13 +496,13 @@ mod tests {
         //   is an int. Navigating to `str` would match pyright's behavior.
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:344:7
+           --> stdlib/builtins.pyi:348:7
             |
-        343 | @disjoint_base
-        344 | class int:
+        347 | @disjoint_base
+        348 | class int:
             |       ^^^
-        345 |     """int([x]) -> integer
-        346 |     int(x, base=10) -> integer
+        349 |     """int([x]) -> integer
+        350 |     int(x, base=10) -> integer
             |
         info: Source
          --> main.py:4:6
@@ -409,13 +529,13 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-            --> stdlib/builtins.pyi:2916:7
+            --> stdlib/builtins.pyi:2920:7
              |
-        2915 | @disjoint_base
-        2916 | class dict(MutableMapping[_KT, _VT]):
+        2919 | @disjoint_base
+        2920 | class dict(MutableMapping[_KT, _VT]):
              |       ^^^^
-        2917 |     """dict() -> new empty dictionary
-        2918 |     dict(mapping) -> new dictionary initialized from a mapping object's
+        2921 |     """dict() -> new empty dictionary
+        2922 |     dict(mapping) -> new dictionary initialized from a mapping object's
              |
         info: Source
          --> main.py:6:5
@@ -439,13 +559,13 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:911:7
+           --> stdlib/builtins.pyi:915:7
             |
-        910 | @disjoint_base
-        911 | class str(Sequence[str]):
+        914 | @disjoint_base
+        915 | class str(Sequence[str]):
             |       ^^^
-        912 |     """str(object='') -> str
-        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        916 |     """str(object='') -> str
+        917 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
          --> main.py:3:5
@@ -531,13 +651,13 @@ f(**kwargs<CURSOR>)
 
         assert_snapshot!(test.goto_type_definition(), @r#"
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:911:7
+           --> stdlib/builtins.pyi:915:7
             |
-        910 | @disjoint_base
-        911 | class str(Sequence[str]):
+        914 | @disjoint_base
+        915 | class str(Sequence[str]):
             |       ^^^
-        912 |     """str(object='') -> str
-        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        916 |     """str(object='') -> str
+        917 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
          --> main.py:4:15
@@ -578,13 +698,13 @@ f(**kwargs<CURSOR>)
           |
 
         info[goto-type-definition]: Type definition
-           --> stdlib/builtins.pyi:911:7
+           --> stdlib/builtins.pyi:915:7
             |
-        910 | @disjoint_base
-        911 | class str(Sequence[str]):
+        914 | @disjoint_base
+        915 | class str(Sequence[str]):
             |       ^^^
-        912 |     """str(object='') -> str
-        913 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
+        916 |     """str(object='') -> str
+        917 |     str(bytes_or_buffer[, encoding[, errors]]) -> str
             |
         info: Source
          --> main.py:3:5
