@@ -1,7 +1,7 @@
 use crate::lint::{GetLintError, Level, LintMetadata, LintRegistry, LintStatus};
 use crate::types::TypeCheckDiagnostics;
 use crate::{Db, declare_lint, lint::LintId};
-use ruff_db::diagnostic::{Annotation, Diagnostic, DiagnosticId, Span};
+use ruff_db::diagnostic::{Annotation, Diagnostic, DiagnosticId, IntoDiagnosticMessage, Span};
 use ruff_db::{files::File, parsed::parsed_module, source::source_text};
 use ruff_python_parser::TokenKind;
 use ruff_python_trivia::Cursor;
@@ -32,7 +32,7 @@ declare_lint! {
     /// ```
     pub(crate) static UNUSED_IGNORE_COMMENT = {
         summary: "detects unused `type: ignore` comments",
-        status: LintStatus::preview("1.0.0"),
+        status: LintStatus::stable("0.0.1-alpha.1"),
         default_level: Level::Ignore,
     }
 }
@@ -55,9 +55,9 @@ declare_lint! {
     /// ```py
     /// a = 20 / 0  # ty: ignore[division-by-zero]
     /// ```
-    pub(crate) static UNKNOWN_RULE = {
+    pub(crate) static IGNORE_COMMENT_UNKNOWN_RULE = {
         summary: "detects `ty: ignore` comments that reference unknown rules",
-        status: LintStatus::preview("1.0.0"),
+        status: LintStatus::stable("0.0.1-alpha.1"),
         default_level: Level::Warn,
     }
 }
@@ -81,12 +81,12 @@ declare_lint! {
     /// ```
     pub(crate) static INVALID_IGNORE_COMMENT = {
         summary: "detects ignore comments that use invalid syntax",
-        status: LintStatus::preview("1.0.0"),
+        status: LintStatus::stable("0.0.1-alpha.1"),
         default_level: Level::Warn,
     }
 }
 
-#[salsa::tracked(returns(ref), heap_size=get_size2::GetSize::get_heap_size)]
+#[salsa::tracked(returns(ref), heap_size=ruff_memory_usage::heap_size)]
 pub(crate) fn suppressions(db: &dyn Db, file: File) -> Suppressions {
     let parsed = parsed_module(db, file).load(db);
     let source = source_text(db, file);
@@ -143,38 +143,12 @@ pub(crate) fn check_suppressions(db: &dyn Db, file: File, diagnostics: &mut Type
 
 /// Checks for `ty: ignore` comments that reference unknown rules.
 fn check_unknown_rule(context: &mut CheckSuppressionsContext) {
-    if context.is_lint_disabled(&UNKNOWN_RULE) {
+    if context.is_lint_disabled(&IGNORE_COMMENT_UNKNOWN_RULE) {
         return;
     }
 
     for unknown in &context.suppressions.unknown {
-        match &unknown.reason {
-            GetLintError::Removed(removed) => {
-                context.report_lint(
-                    &UNKNOWN_RULE,
-                    unknown.range,
-                    format_args!("Removed rule `{removed}`"),
-                );
-            }
-            GetLintError::Unknown(rule) => {
-                context.report_lint(
-                    &UNKNOWN_RULE,
-                    unknown.range,
-                    format_args!("Unknown rule `{rule}`"),
-                );
-            }
-
-            GetLintError::PrefixedWithCategory {
-                prefixed,
-                suggestion,
-            } => {
-                context.report_lint(
-                    &UNKNOWN_RULE,
-                    unknown.range,
-                    format_args!("Unknown rule `{prefixed}`. Did you mean `{suggestion}`?"),
-                );
-            }
-        }
+        context.report_lint(&IGNORE_COMMENT_UNKNOWN_RULE, unknown.range, &unknown.reason);
     }
 }
 
@@ -300,7 +274,7 @@ impl<'a> CheckSuppressionsContext<'a> {
         &mut self,
         lint: &'static LintMetadata,
         range: TextRange,
-        message: fmt::Arguments,
+        message: impl IntoDiagnosticMessage,
     ) {
         if let Some(suppression) = self.suppressions.find_suppression(range, LintId::of(lint)) {
             self.diagnostics.mark_used(suppression.id());
@@ -316,7 +290,7 @@ impl<'a> CheckSuppressionsContext<'a> {
         &mut self,
         lint: &'static LintMetadata,
         range: TextRange,
-        message: fmt::Arguments,
+        message: impl IntoDiagnosticMessage,
     ) {
         let Some(severity) = self.db.rule_selection(self.file).severity(LintId::of(lint)) else {
             return;
@@ -324,6 +298,7 @@ impl<'a> CheckSuppressionsContext<'a> {
 
         let id = DiagnosticId::Lint(lint.name());
         let mut diag = Diagnostic::new(id, severity, "");
+        diag.set_documentation_url(Some(lint.documentation_url()));
         let span = Span::from(self.file).with_range(range);
         diag.annotate(Annotation::primary(span).message(message));
         self.diagnostics.push(diag);
@@ -402,7 +377,7 @@ impl Suppressions {
             })
     }
 
-    fn iter(&self) -> SuppressionsIter {
+    fn iter(&self) -> SuppressionsIter<'_> {
         self.file.iter().chain(&self.line)
     }
 }
@@ -508,7 +483,7 @@ impl<'a> SuppressionsBuilder<'a> {
             lint_registry,
             seen_non_trivia_token: false,
             line: Vec::new(),
-            file: SmallVec::new(),
+            file: SmallVec::new_const(),
             unknown: Vec::new(),
             invalid: Vec::new(),
         }
