@@ -28,6 +28,7 @@ import {
   DocumentHighlight,
   DocumentHighlightKind,
   InlayHintKind,
+  LocationLink,
   TextEdit,
 } from "ty_wasm";
 import { FileId, ReadonlyFiles } from "../Playground";
@@ -70,13 +71,16 @@ export default function Editor({
   const serverRef = useRef<PlaygroundServer | null>(null);
 
   if (serverRef.current != null) {
-    serverRef.current.update({
-      files,
-      workspace,
-      onOpenFile,
-      onVendoredFileChange,
-      onBackToUserFile,
-    });
+    serverRef.current.update(
+      {
+        files,
+        workspace,
+        onOpenFile,
+        onVendoredFileChange,
+        onBackToUserFile,
+      },
+      isViewingVendoredFile,
+    );
   }
 
   // Update the diagnostics in the editor.
@@ -200,6 +204,7 @@ class PlaygroundServer
   private rangeSemanticTokensDisposable: IDisposable;
   private signatureHelpDisposable: IDisposable;
   private documentHighlightDisposable: IDisposable;
+  private inVendoredFileCondition: editor.IContextKey<boolean>;
   // Cache for vendored file handles
   private vendoredFileHandles = new Map<string, FileHandle>();
 
@@ -249,8 +254,16 @@ class PlaygroundServer
     this.documentHighlightDisposable =
       monaco.languages.registerDocumentHighlightProvider("python", this);
 
+    this.inVendoredFileCondition = editor.createContextKey<boolean>(
+      "inVendoredFile",
+      false,
+    );
     // Register Esc key command
-    editor.addCommand(monaco.KeyCode.Escape, this.props.onBackToUserFile);
+    editor.addCommand(
+      monaco.KeyCode.Escape,
+      () => this.props.onBackToUserFile(),
+      "inVendoredFile",
+    );
   }
 
   triggerCharacters: string[] = ["."];
@@ -433,7 +446,10 @@ class PlaygroundServer
     return {
       dispose: () => {},
       hints: inlayHints.map((hint) => ({
-        label: hint.markdown,
+        label: hint.label.map((part) => ({
+          label: part.label,
+          // As of 2025-09-23, location isn't supported by Monaco which is why we don't set it
+        })),
         position: {
           lineNumber: hint.position.line,
           column: hint.position.column,
@@ -452,8 +468,9 @@ class PlaygroundServer
     return undefined;
   }
 
-  update(props: PlaygroundServerProps) {
+  update(props: PlaygroundServerProps, isViewingVendoredFile: boolean) {
     this.props = props;
+    this.inVendoredFileCondition.set(isViewingVendoredFile);
   }
 
   private getOrCreateVendoredFileHandle(vendoredPath: string): FileHandle {
@@ -750,57 +767,59 @@ class PlaygroundServer
     return null;
   }
 
-  private mapNavigationTargets(links: any[]): languages.LocationLink[] {
-    const result = links.map((link) => {
-      const uri = Uri.parse(link.path);
+  private mapNavigationTarget(link: LocationLink): languages.LocationLink {
+    const uri = Uri.parse(link.path);
 
-      // Pre-create models to ensure peek definition works
-      if (this.monaco.editor.getModel(uri) == null) {
-        if (uri.scheme === "vendored") {
-          // Handle vendored files
-          const vendoredPath = this.getVendoredPath(uri);
-          const fileHandle = this.getOrCreateVendoredFileHandle(vendoredPath);
-          const content = this.props.workspace.sourceText(fileHandle);
-          this.monaco.editor.createModel(content, "python", uri);
-        } else {
-          // Handle regular files
-          const fileId = this.props.files.index.find((file) => {
-            return Uri.file(file.name).toString() === uri.toString();
-          })?.id;
+    // Pre-create models to ensure peek definition works
+    if (this.monaco.editor.getModel(uri) == null) {
+      if (uri.scheme === "vendored") {
+        // Handle vendored files
+        const vendoredPath = this.getVendoredPath(uri);
+        const fileHandle = this.getOrCreateVendoredFileHandle(vendoredPath);
+        const content = this.props.workspace.sourceText(fileHandle);
+        this.monaco.editor.createModel(content, "python", uri);
+      } else {
+        // Handle regular files
+        const fileId = this.props.files.index.find((file) => {
+          return Uri.file(file.name).toString() === uri.toString();
+        })?.id;
 
-          if (fileId != null) {
-            const handle = this.props.files.handles[fileId];
-            if (handle != null) {
-              const language = isPythonFile(handle) ? "python" : undefined;
-              this.monaco.editor.createModel(
-                this.props.files.contents[fileId],
-                language,
-                uri,
-              );
-            }
+        if (fileId != null) {
+          const handle = this.props.files.handles[fileId];
+          if (handle != null) {
+            const language = isPythonFile(handle) ? "python" : undefined;
+            this.monaco.editor.createModel(
+              this.props.files.contents[fileId],
+              language,
+              uri,
+            );
           }
         }
       }
+    }
 
-      const targetSelection =
-        link.selection_range == null
-          ? undefined
-          : tyRangeToMonacoRange(link.selection_range);
+    const targetSelection =
+      link.selection_range == null
+        ? undefined
+        : tyRangeToMonacoRange(link.selection_range);
 
-      const originSelection =
-        link.origin_selection_range == null
-          ? undefined
-          : tyRangeToMonacoRange(link.origin_selection_range);
+    const originSelection =
+      link.origin_selection_range == null
+        ? undefined
+        : tyRangeToMonacoRange(link.origin_selection_range);
 
-      return {
-        uri: uri,
-        range: tyRangeToMonacoRange(link.full_range),
-        targetSelectionRange: targetSelection,
-        originSelectionRange: originSelection,
-      } as languages.LocationLink;
-    });
+    return {
+      uri: uri,
+      range: tyRangeToMonacoRange(link.full_range),
+      targetSelectionRange: targetSelection,
+      originSelectionRange: originSelection,
+    } as languages.LocationLink;
+  }
 
-    return result;
+  private mapNavigationTargets(
+    links: LocationLink[],
+  ): languages.LocationLink[] {
+    return links.map((link) => this.mapNavigationTarget(link));
   }
 
   dispose() {
